#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Function:Install license
Date:2018.12.13
"""
import sys
import time

CONTENT_HELP = '''Content of license.
License text content if type is 'Text' while license file uri if type is 'URI'.
License file URI supports both bmc local path URI (only under the /tmp directory), 
and remote path URI (protocols HTTPS, SFTP, NFS, CIFS, and SCP are supported).
'''

TYPE_HELP = '''Content type.
Text: indicates the value is the license content.
URI: indicates the value is URI (local path or remote path).
'''


def installlicense_init(parser, parser_list):
    """
    :Function:Install license subcommand
    :param parser:major command argparser
    :param parser_list:save subcommand parser list
    :return:
    """
    sub_parser = parser.add_parser('installlicense',
                                   help='''install license''')
    sub_parser.add_argument('-S', dest='source',
                            default="iBMC",
                            choices=['iBMC', 'FusionDirector', 'eSight'],
                            help='License source, iBMC by default.')
    sub_parser.add_argument('-T', dest='type',
                            required=True,
                            choices=['URI', 'Text'],
                            help=TYPE_HELP)
    sub_parser.add_argument('-C', dest='content',
                            type=str,
                            required=True,
                            help=CONTENT_HELP)

    parser_list['installlicense'] = sub_parser
    return 'installlicense'


def installlicense(client, parser, args):
    """
    :Function:Install license
    :param client:RedfishClient object
    :param parser:subcommand argparser. Export error messages when parameters are incorrect.
    :param args:parameter list
    :return:
    """
    # Parameter verification
    if parser is None or args is None:
        return None

    # Obtain the slot number.
    slotid = client.get_slotid()
    if slotid is None:
        return None

    url = "/redfish/v1/Managers/%s/LicenseService/Actions" \
          "/LicenseService.InstallLicense" % slotid
    # Construct payload.
    payload = {
        "FileSource": args.source,
        "Type": args.type,
        "Content": args.content
    }
    resp = client.create_resource(url, payload)

    if resp is None:
        return None

    if resp.get('status_code') == 202:
        time.sleep(1)
        resp_task = client.print_task_prog(resp)
        if resp_task is None:
            return None

        if resp_task == 'Exception':
            _resptaskparse(resp, client)
            sys.exit(144)
    else:
        _respparse(resp)

    return resp


def _resptaskparse(resp, client):
    """
    :Function:Handle exception task state
    :param client:RedfishClient object
    :param resp:response information
    :return:
    """

    taskid = resp['resource']['@odata.id']

    sys_resp = client.get_resource(taskid)
    if sys_resp is None:
        sys.exit(127)

    if sys_resp['status_code'] != 200:
        message = (sys_resp['message']['error']['@Message.ExtendedInfo'][0]['Message']).lower()
        print 'Failure: ' + message[:-1]
    else:
        # Return the task failure details
        message = (sys_resp['resource']['Messages']['Message']).lower()
        print 'Failure: ' + message[:-1]


def _respparse(resp):
    """
    :Function:Handle resp which is not 202
    :param resp:response information
    :return:
    """
    if resp['status_code'] == 200:
        print 'Success: successfully completed request'
        return None
    else:
        extended_info = resp['message']['error']['@Message.ExtendedInfo'][0]
        message_id = extended_info.get('MessageId')
        if 'Format' in message_id:
            print 'Failure: import failed due to invalid path'
        else:
            message = (extended_info.get('Message')).lower()
            print 'Failure: ' + message[:-1]
